import os
from argparse import Namespace
from copy import copy
from pathlib import Path

from luscious_dl.logger import logger_file_handler, logger
from luscious_dl.parser import is_a_valid_integer
from luscious_dl.start import start
from luscious_dl.utils import cls, create_default_files, open_config_menu, get_config_data, read_list, info, \
  ListFilesManager, inputs_string_to_list


def list_txt_organizer(items: list[str], prefix: str) -> None:
  """
  Remove from list.txt and then add to list_completed.txt
  :param items: List of urls or ids
  :param prefix: album/user
  """
  for item in items:
    ListFilesManager.remove(item)
    ListFilesManager.add(f'{prefix}-{int(item)}' if is_a_valid_integer(item) else item)


def menu() -> None:
  """Menu"""
  info()
  create_default_files()
  logger_file_handler()
  configs = get_config_data()
  base_namespace = Namespace(
    output_dir=Path(os.path.normcase(configs.get('directory', './albums/'))).resolve(),
    threads=configs.get('pool', os.cpu_count() or 1),
    retries=configs.get('retries', 5),
    timeout=configs.get('timeout', 30),
    delay=configs.get('delay', 0),
    foldername_format=configs.get('foldername_format', '%t'),
    gen_pdf=configs.get('gen_pdf', False),
    rm_origin_dir=configs.get('rm_origin_dir', False),
    group_by_user=configs.get('group_by_user', False),
    album_inputs=None, user_inputs=None, only_favorites=False,
    keyword=None, search_download=False, sorting='date_trending', page=1, max_pages=1)

  while True:
    option = input('Options:\n'
                   '1 - Download albums by URL or ID.\n'
                   '2 - Download all user albums.\n'
                   '3 - Download all user favorites.\n'
                   '4 - Search albums by keyword.\n'
                   '5 - Download albums from list.txt.\n'
                   '6 - Settings.\n'
                   '0 - Exit.\n'
                   '> ')

    if option in ('1', '2', '3'):
      inputs = input('\n0 - Back.\n'
                     f'Enter {"album" if option == "1" else "user"} URL or ID.\n> ')
      cls()
      if inputs != '0':
        args = copy(base_namespace)
        args.album_inputs = inputs if option == '1' else None
        args.user_inputs = inputs if option in ('2', '3') else None
        args.only_favorites = option == '3'
        start(args)
        list_txt_organizer(inputs_string_to_list(inputs), 'album' if option == '1' else 'user')
        logger.log(5, 'URLs/IDs added to completed list.')

    elif option == '4':
      keyword = input('Enter keyword\n> ')
      if not keyword:
        print('Please enter a keyword.\n')
        return
      page = input('Enter starting page number or leave blank\n> ')
      page = int(page) if is_a_valid_integer(page) else 1
      max_pages = input('Enter max page or leave blank\n> ')
      max_pages = int(max_pages) if is_a_valid_integer(max_pages) else 1
      search_download = input('Download search results? ("Y/N") ').strip() in 'yY'
      args = copy(base_namespace)
      args.keyword = keyword
      args.search_download = search_download
      args.page = page
      args.max_pages = max_pages
      start(args)

    elif option == '5':
      list_txt = list(set(read_list()))
      args = copy(base_namespace)
      args.album_inputs = ','.join(list_txt)
      start(args)
      list_txt_organizer(list_txt, 'album')
      logger.log(5, 'URLs/IDs added to completed list.')

    elif option == '6':
      open_config_menu()

    elif option == '0':
      exit()

    else:
      print('Invalid Option.\n')


if __name__ == '__main__':
  menu()



import os
from argparse import Namespace
from pathlib import Path

from luscious_dl.album import Album, search_albums, print_search
from luscious_dl.command_line import command_line
from luscious_dl.downloader import Downloader
from luscious_dl.logger import logger
from luscious_dl.parser import extract_ids_from_list, extract_album_id, extract_user_id
from luscious_dl.user import User
from luscious_dl.utils import info, format_foldername, generate_pdf, inputs_string_to_list


def albums_download(albums_ids: list[int], downloader: Downloader, output_dir: Path,
                    foldername_format='%t', gen_pdf=False, rm_origin_dir=False) -> None:
  """
  Start albums download process.
  :param albums_ids: list of album ids
  :param downloader: Downloader object
  :param foldername_format: album folder name format
  :param output_dir: Path where albums will be saved
  :param gen_pdf: indicates whether to generate the pdf file
  :param rm_origin_dir: indicates whether the source folder will be deleted
  """
  for id_ in albums_ids:
    album = Album(id_)
    try:
      if album.fetch_info():
        formmatted_foldername = format_foldername(album, foldername_format)
        album_folder = Path.joinpath(output_dir, formmatted_foldername)
        album.show()
        album.fetch_pictures()
        # album.generate_metadata(album_folder)
        album.download(downloader, album_folder)
        if gen_pdf:
          generate_pdf(output_dir, formmatted_foldername, album_folder, rm_origin_dir)
      else:
        raise Exception('Album Information not found.')
    except Exception as e:
      logger.critical(f'Album: {id_} Error: {e}')


def users_download(users_ids: list[int], downloader: Downloader, output_dir: Path, foldername_format='%t',
                   only_favorites=False, gen_pdf=False, rm_origin_dir=False, group_by_user=False) -> None:
  """
  Start users download process.
  :param users_ids: list of user ids
  :param downloader: Downloader object
  :param output_dir: Path where albums will be saved
  :param foldername_format: album folder name format
  :param only_favorites: defines if it's to download only the favorites
  :param gen_pdf: whether to generate the pdf file
  :param rm_origin_dir: indicates whether the source folder will be deleted
  :param group_by_user:
  """
  for id_ in users_ids:
    user = User(id_)
    try:
      if user.fetch_info():
        user.fetch_albums(only_favorites)
        user.show()
        if group_by_user:
          output_dir = output_dir.joinpath(os.path.normcase(user.name))
        albums_download(user.albums_ids, downloader, output_dir, foldername_format, gen_pdf, rm_origin_dir)
      else:
        raise Exception('User Information not found.')
    except Exception as e:
      logger.critical(f'User: {id_} Error: {e}')


def normalize_args(args: Namespace) -> Namespace:
  """
  Fix possible args inconsistencies.
  :param args: Namespace
  :return: Fixed Namespace
  """
  if args.threads <= 0:
    args.threads = os.cpu_count()
    if not args.threads:
      logger.warning('It was not possible to determine the number of CPUs in your system. '
                     'Only one will be used, this will decrease the amount of downloads.')
      args.threads = 1
  if args.page <= 0:
    args.page = 1
  if args.max_pages <= 0:
    args.max_pages = 1
  if args.page > args.max_pages:
    args.max_pages = args.page

  if args.only_favorites and not args.user_inputs:
    logger.warn(f"You're passing --favorites/-f flag without any user input.")
    args.only_favorites = False

  if args.gen_pdf and not args.album_inputs and not args.user_inputs and not args.search_download:
    logger.warn(f"You're passing --pdf flag without any album/user input or search download.")
    args.gen_pdf = False

  args.keyword = args.keyword.strip() if args.keyword else None

  if args.album_inputs:
    inputs = inputs_string_to_list(args.album_inputs)
    args.albums_ids = extract_ids_from_list(inputs, extract_album_id)
  else:
    args.albums_ids = None

  if args.user_inputs:
    inputs = inputs_string_to_list(args.user_inputs)
    args.users_ids = extract_ids_from_list(inputs, extract_user_id)
  else:
    args.users_ids = None

  return args


def start(args: Namespace = None) -> None:
  """Start"""
  if not args:
    info()
  args = normalize_args(args or command_line())
  downloader = Downloader(args.threads, args.retries, args.timeout, args.delay)

  if args.albums_ids:
    albums_download(args.albums_ids, downloader, args.output_dir, args.foldername_format,
                    args.gen_pdf, args.rm_origin_dir)
  elif args.users_ids:
    users_download(args.users_ids, downloader, args.output_dir, args.foldername_format,
                   args.only_favorites, args.gen_pdf, args.rm_origin_dir, args.group_by_user)
  elif args.keyword:
    result = search_albums(args.keyword, args.sorting, args.page, args.max_pages)
    print_search(result)
    if args.search_download:
      for album in result:
        album.show()
        album.fetch_pictures()
        formmatted_foldername = format_foldername(album, args.foldername_format)
        album_folder = Path.joinpath(args.output_dir, formmatted_foldername)
        album.download(downloader, album_folder)
        if args.gen_pdf:
          generate_pdf(args.output_dir, formmatted_foldername, album_folder, args.rm_origin_dir)


if __name__ == '__main__':
  start()



def album_info_query(album_id: str) -> dict:
  """
  Build Album information query.
  :param album_id: Album id
  :return: Query
  """
  return {
    "operationName": "AlbumGet",
    "query": """query AlbumGet($id: ID!) {
                  album {
                    get(id: $id) {
                      ... on Album {...AlbumStandard}
                      ... on MutationError {errors {code message}}
                    }
                  }
                }
                fragment AlbumStandard on Album {
                  id
                  title
                  slug
                  url
                  description
                  created_by {
                    name
                    display_name
                  }
                  number_of_pictures
                  number_of_animated_pictures
                  language {
                    title
                  }
                  tags {
                    text
                  }
                  genres {
                    title
                  }
                  audiences {
                    title
                  }
                }
    """,
    "variables": {
      "id": album_id
    }
  }


def album_list_pictures_query(album_id: str, page_number: int) -> dict:
  """
  Build Album pictures query.
  :param album_id: Album id
  :param page_number: initial search page
  :return: Query
  """
  return {
    "operationName": "AlbumListOwnPictures",
    "query": """query AlbumListOwnPictures($input: PictureListInput!) {
                  picture {
                    list(input: $input) {
                      info {...FacetCollectionInfo}
                      items {...PictureStandardWithoutAlbum}
                    }
                  }
                }
                fragment FacetCollectionInfo on FacetCollectionInfo {
                  page
                  has_next_page
                  has_previous_page
                  total_items
                  total_pages
                  items_per_page
                }
                fragment PictureStandardWithoutAlbum on Picture {
                  url_to_original
                  url_to_video
                  url
                }
    """,
    "variables": {
      "input": {
        "filters": [
          {
            "name": "album_id",
            "value": album_id
          }
        ],
        "display": "rating_all_time",
        "page": page_number
      }
    }
  }


def album_search_query(search_query: str, sorting: str = 'date_trending', page: int = 1) -> dict:
  """
  Build Album search query.
  :param search_query: keyword
  :param sorting:
  :param page: initial search page
  :return: Query
  """
  return {
    "operationName": "AlbumList",
    "query": """query AlbumList($input: AlbumListInput!) {
                  album {
                    list(input: $input) {
                      info {...FacetCollectionInfo}
                      items {...AlbumMinimal}
                    }
                  }
                }
                fragment FacetCollectionInfo on FacetCollectionInfo {
                  page
                  has_next_page
                  has_previous_page
                  total_items
                  total_pages
                  items_per_page
                }
                fragment AlbumMinimal on Album {
                  __typename
                  id
                  title
                  number_of_pictures
                  number_of_animated_pictures
                  created_by {
                    id
                    url
                    name
                    display_name
                    user_title
                  }
                }
    """,
    "variables": {
      "input": {
        "display": sorting,
        "filters": [
          {
            "name": "album_type",
            "value": "pictures"
          },
          {
            "name": "search_query",
            "value": search_query
          }
        ],
        "page": page
      }
    }
  }


def user_albums_query(user_id: str, page_number: int) -> dict:
  """
  Build User albums query.
  :param user_id: User id
  :param page_number: page number
  :return: Query
  """
  return {
    "operationName": "AlbumList",
    "query": """query AlbumList($input: AlbumListInput!) {
                  album {
                    list(input: $input) {
                      info {...FacetCollectionInfo}
                      items {...AlbumMinimal}
                    }
                  }
                }
                fragment FacetCollectionInfo on FacetCollectionInfo {
                  page
                  has_next_page
                  has_previous_page
                  total_items
                  total_pages
                  url_complete
                }
                fragment AlbumMinimal on Album {
                  id
                }
    """,
    "variables": {
      "input": {
        "display": "date_newest",
        "filters": [
          {
            "name": "created_by_id",
            "value": user_id
          }
        ],
        "page": page_number
      }
    }
  }


def user_info_query(user_id: str) -> dict:
  """
  Build User information query.
  :param user_id: User id
  :return: Query
  """
  return {
    "operationName": "ProfileGet",
    "query": """query ProfileGet($user_id: ID!) {
                  userprofile {
                    get(user_id: $user_id) {
                      ... on UserProfile {
                        id 
                        user {
                          id
                          name
                          display_name
                          user_title
                        }
                        number_of_posts
                        number_of_albums
                        number_of_videos
                        number_of_favorite_albums
                        is_banned
                        filter_settings {
                          uses_default_warnings
                          audience_ids
                          genres_blocked_ids
                          genres_subscribed_ids
                        }
                      }
                      ... on MutationError {errors {code message}}
                    }
                  }
                }
    """,
    "variables": {
      "user_id": user_id
    }
  }


def user_favorites_query(user_id: str, page_number: int) -> dict:
  """
  Build User favorites query.
  :param user_id: User id
  :param page_number: page number
  :return: Query
  """
  return {
    "operationName": "AlbumList",
    "query": """query AlbumList($input: AlbumListInput!) {
                  album {
                    list(input: $input) {
                      info {
                        ...FacetCollectionInfo}items {...AlbumMinimal}
                      }
                    }
                  }
                  fragment FacetCollectionInfo on FacetCollectionInfo {
                    page
                    has_next_page
                    has_previous_page
                    total_items
                    total_pages
                    url_complete
                  }
                  fragment AlbumMinimal on Album {
                    id
                  }
    """,
    "variables": {
      "input": {
        "display": "date_newest",
        "filters": [
          {
            "name": "favorite_by_user_id",
            "value": user_id
          }
        ],
        "page": page_number
      }
    }
  }